package SOMA.agent.mobility;

import SOMA.agent.*;
import SOMA.agent.classLoading.*;
import SOMA.naming.*;
import SOMA.explorer.*;
import SOMA.Environment;
import SOMA.utility.IndexHashtable;
import SOMA.security.*;
import SOMA.security.infrastructure.*;

import java.util.*;
import java.io.*;
import java.io.PrintStream;

/** Gestore degli {@link SOMA.agent.Agent agenti} di un {@link SOMA.Environment place}.
*
* @author Luca Ghetti
*/
public class AgentManagerSecurity extends AgentManager
{/*
  Environment env;
  AgentSystem agentSystem;

  public ClassManager agentClassManager;
  public ClassManager cacheClassManager;
  public IndexHashtable indexStore = new IndexHashtable();

  StringExplorerItem path, cache;

  public int AgentIDCounter = 1;

  public AgentWorkerStore agentWorkerStore = new AgentWorkerStore();
  public AgentPositionStore agentPositionStore = new AgentPositionStore();
  DirExplorerItem agentManagerDir;
  public final int MAX_MESSAGE_ATTEMPTS = 6;
*/
  /** Costruttore.
  * @param env L'environment del place.
  */
  public AgentManagerSecurity( Environment env )
  {
    super();

    this.env = env ;

    agentSystem = new AgentSystem( env );

    agentManagerDir = new DirExplorerItem( "agentManager" );
    env.dir.addItem( agentManagerDir );

    // Entrambe NON creano la directory se non esiste gia'
    try
    {
      agentClassManager = new ClassManager( System.getProperty( "user.dir" ) + File.separator + "agents", false );
      cacheClassManager = new ClassManager( System.getProperty( "user.dir" ) + File.separator + "cache", false );
    }
    catch( Exception e )
    {
      e.printStackTrace( env.err );
    }

    path = new ClassManagerExplorerItem( agentClassManager )
      {
        public void update( ClassManager classManager )
        {
          agentClassManager = classManager;
        }
      };

    cache = new ClassManagerExplorerItem( cacheClassManager )
      {
        public void update( ClassManager classManager )
        {
          cacheClassManager = classManager;
        }
      };

    agentManagerDir.addItem( "path", path );
    agentManagerDir.addItem( "cache", cache );
    agentManagerDir.addItem( "store", new ObjectExplorerItem( indexStore ) );

    agentManagerDir.addItem( "list", new ExplorerItem( "List of agent workers" )
      {
        public Object Execute( Collection Parameters, PrintStream out )
        {
          agentWorkerStore.printWorkers( out );
          return null;
        }
      });
    agentManagerDir.addItem( "a" , new AgentWorkerExplorerItem( env ) );

    agentManagerDir.addItem( "pos" , new ExplorerItem( "List of agents positions." )
      {
        public Object Execute( Collection Parameters, PrintStream out )
        {
          agentPositionStore.printPositions( out );
          return null;
        }
      });

    agentManagerDir.addItem( "death", new ExplorerItem( "\"Agent ID\"" )
      {
        public Object Execute( Collection Parameters, PrintStream out )
        {
          try
          {
            agentDeath( new AgentID( (String)Parameters.iterator().next() ) );
            out.println( "DONE!" );
          }
          catch( Exception e )
          {
            e.printStackTrace();
          }
          return null;
        }

        public String Help( PrintStream out )
        {
          out.println( "Notifies an agent's death to his home place." );
          out.println( "  Use it only if the agent is really dead!" );

          return "Notifies an agent's death to his home place.";
        }
      });

    agentManagerDir.addItem( "save", new ExplorerItem( "Save agents to disk." )
      {
        public Object Execute( java.util.Collection Parameters, PrintStream out )
        {
          try
          {
            save();
            out.println( "Saved!" );
          }
          catch( Exception e )
          {
            e.printStackTrace( out );
          }

          return null;
        }
      } );

    agentManagerDir.addItem( "load", new ExplorerItem( "Load agents from disk." )
      {
        public Object Execute( java.util.Collection Parameters, PrintStream out )
        {
          try
          {
            load();
            out.println( "Loaded!" );
          }
          catch( Exception e )
          {
            e.printStackTrace( out );
          }

          return null;
        }
      } );

    /*
    agentManagerDir.addItem( "launchSys" , new ExplorerItem( "<Agent> { <Arg1> <Arg2> ... }" )
      {
        public Object Execute( Collection Parameters, PrintStream out )
        {
          int NumParameters = Parameters.size();

          if( NumParameters > 0 )
          {
            Iterator i = Parameters.iterator();

            String agentName = (String)i.next();

            String[] ParametersArray = new String[NumParameters - 1];

            out.println( "Creating agent: " + agentName + " Parameters: " );

            for( int j = 0; j < NumParameters - 1; j++ )
            {
              ParametersArray[j] = (String)i.next();
              out.println( "  " + j + ")  " + ParametersArray[j] );
            }

            // true --> System agent!
            AgentID created = createAgent( agentName, ParametersArray, true, true );

            if( created == null )
              out.println( "ERROR: can't create agent." );

            return created;
          }

          out.println( "incorrect number of parameters" );
          return null;
        }
      });

    agentManagerDir.addItem( "launchNoTra" , new ExplorerItem( "<Agent> { <Arg1> <Arg2> ... }" )
      {
        public Object Execute( Collection Parameters, PrintStream out )
        {
          int NumParameters = Parameters.size();

          if( NumParameters > 0 )
          {
            Iterator i = Parameters.iterator();

            String agentName = (String)i.next();

            String[] ParametersArray = new String[NumParameters - 1];

            out.println( "Creating agent: " + agentName + " Parameters: " );

            for( int j = 0; j < NumParameters - 1; j++ )
            {
              ParametersArray[j] = (String)i.next();
              out.println( "  " + j + ")  " + ParametersArray[j] );
            }

            // false --> System agent!
            // false --> Not traceable!
            AgentID created = createAgent( agentName, ParametersArray, false, false );

            if( created == null )
              out.println( "ERROR: can't create agent." );

            return created;
          }

          out.println( "incorrect number of parameters" );
          return null;
        }
      });


    ExplorerItem launch = new ExplorerItem( "<Agent> { <Arg1> <Arg2> ... }" )
      {
        public Object Execute( Collection Parameters, PrintStream out )
        {
          int NumParameters = Parameters.size();

          if( NumParameters > 0 )
          {
            Iterator i = Parameters.iterator();

            String agentName = (String)i.next();

            String[] ParametersArray = new String[NumParameters - 1];

            out.println( "Creating agent: " + agentName + " Parameters: " );

            for( int j = 0; j < NumParameters - 1; j++ )
            {
              ParametersArray[j] = (String)i.next();
              out.println( "  " + j + ")  " + ParametersArray[j] );
            }

            // false --> NON System agent!
            AgentID created = createAgent( agentName, ParametersArray, false, true );

            if( created == null )
              out.println( "ERROR: can't create agent." );

            return created;
          }

          out.println( "incorrect number of parameters" );
          return null;
        }
      };
    */

    ExplorerItem launch = new AgentLauncherExplorerItemSecurity( env );
    agentManagerDir.addItem( "launch", launch );
    env.dir.addItem( "launch", launch );
  }

  /** Creazione di un agente di sistema.
  *
  * @param agentName Nome dell'agente.
  * @param argument Parametro di inizializxzazione, vedi
  * {@link SOMA.agent.Agent#putArgument( Object obj )}.
  */
  /*
  public AgentID createSystemAgent( String agentName, Object argument )
  {
    AgentID returnID = null;

    try
    {
      Agent agent = (Agent)Class.forName( agentName ).newInstance();
      agent.setID( newAgentID() );
      agent.putArgument( argument );
      //env.out.println( "AgentManager.createAgent: agent " + agent + "CREATED" );

      returnID = startAgent( agent );
    }
    catch( Exception e )
    {
      e.printStackTrace( env.err );
    }

    return returnID;
  }*/

  /** Creazione di un agente.
  *
  * @param agentName Nome dell'agente.
  * @param argument Parametro di inizializzazione, vedi
  * {@link SOMA.agent.Agent#putArgument( Object obj )}.
  * @param isSystemAgent Se a true si forza l'utilizzo del classloader di sistema
  * @param traceable Se a true l'agente ha un  traceable ed ha una mailbox.
  */
  public AgentWorker createAgent( String agentName, Object argument, boolean isSystemAgent, boolean traceable , ProfileManager profile)
  {
    //AgentID returnID = null;
    AgentWorker worker = null;

    try
    {
      AgentID newID = newAgentID();

      if (profile != null )
      {
        ((AgentIDSigned)newID).signatureState(profile);
      }

      if (profile != null )
      {
        ((AgentIDSigned)newID).signatureCode(profile, agentName);
      }

      ClassLoader classLoader;

      if( isSystemAgent )
        classLoader = getClass().getClassLoader(); // Lo stesso ClassLoader della classe attuale!
      else
        classLoader = new AgentClassLoaderSecurity( env, agentName, newID , true );


      Agent agent = (Agent)classLoader.loadClass( agentName ).newInstance();
      agent.setID( newID );
      agent.putArgument( argument );
      agent.setTraceable( traceable );

      /* se faccio la firma qui al primo caricamento della classe non ho alcuna operazione di verifica
      if (profile != null )
      {
        agent.signatureCode(profile);
      }

      if (profile != null )
      {
        agent.signatureState(profile);
      }
       */

      //env.out.println( "AgentManager.createAgent: agent " + agent + "CREATED by" + agent.getClass().getClassLoader() );

      worker = createWorker( agent );
    }
    catch( Exception e )
    {
      e.printStackTrace( env.err );
    }

    return worker;
    //return returnID;
  }


  /** Creazione del worker di un agente.
  *
  * <P> Prima di creare il worker si verificano i sui diritti
  * di accesso al place {@link PlaceAccessPermission}.
  */
  public AgentWorker createWorker( Agent agent )
  {
    //AgentID returnID = null;
    AgentWorker worker = null;
    try
    {
      // Se non c' il security manager, non effettuo neanche i controlli.

      if( System.getSecurityManager() == null ||
          agent.getClass().getProtectionDomain().implies( new PlaceAccessPermission( env.placeID ) ) )
      {
        agent.agentSystem = agentSystem;
        worker = new AgentWorker( agent, env );
        agentWorkerStore.putWorker( worker );

        //env.out.println( "AgentManager.createAgent: creato e memorizzato worker " + worker );
        //worker.start();
        //env.out.println( "AgentManager.createAgent: AVVIATO worker " + worker );

        // Aggiorno la posizione dell'agente: attenzione: posso spedire un comando anche nel place in cui mi trovo
        //   l'operazione ha costo bassissimo (SelfConnection).
        if( agent.traceable )
          env.networkManager.sendCommand( agent.getID().getHome(), new AgentPositionUpdateCommand( agent.getID(), env.placeID ));

        //returnID = worker.agent.getID();
      }
      else
        env.err.println( "AgentManager.createWorker: Agent " + agent + " has no PlaceAccessPermission to " + env.placeID );
    }
    catch( Exception e )
    {
      e.printStackTrace( env.err );
    }

    return worker;
    //return returnID;
  }

 /** Spedizione di un messaggio ad un altro agente.
  *
  * <P> Non e' possibile statilire se il messaggio sara' correttamente recapitato.
  *
  * <P>Questo metodo si interfaccia direttamente al {@link SOMA.network.NetworkManager}.
  */
  public void sendMessage( Message message )
  {
    sendMessage( message, 0 );
  }


  public synchronized void sendMessage( Message message, int attemptsCount )
  {
    AgentWorker destinationWorker =
       env.agentManager.agentWorkerStore.getWorker( message.to );

    if( destinationWorker != null &&
        destinationWorker.getStatus() != AgentWorker.GONE ) // Ho gi trovato l'agente!
    {
      destinationWorker.agent.mailbox.storeMessage( message );
      //env.out.println( "MESSAGE " + message + " delivered." );

      // ATTENZIONE: Agenti in idle ==> riavviati.
      if( destinationWorker.getStatus() == AgentWorker.OFF ||
          destinationWorker.getStatus() == AgentWorker.IDLE ||
          destinationWorker.getStatus() == AgentWorker.STOPPED )
      {
        env.out.println( "Waking up agent " + message.to );
        try
        {
          destinationWorker.start();
        }
        catch( AgentWorker.AgentWorkerException e )
        {
          e.printStackTrace();
        }
      }
    }
    else if( attemptsCount++ < MAX_MESSAGE_ATTEMPTS ) // Numero massimo di tentativi
    {  // se destinationWorker = null agente non e' locale
      SendMessageCommand sendMessageCommand = new SendMessageCommand( message, attemptsCount );

      // Trovo in quale place  scritta la sua posizione attuale.
      if( env.placeID.equals( message.to.getHome() ))
             // mi trovo nella Home dell'Agente: trovo la posizione dell'agente
      {
        //2) Qui c' scritto dov' l'agente?
        PlaceID currentAgentPosition = env.agentManager.agentPositionStore.get( message.to );

        if( currentAgentPosition != null )
        {
          env.networkManager.sendCommand( currentAgentPosition, sendMessageCommand );

          //env.err.println("MESSAGE " + message  + " sent to: " + currentAgentPosition );
        }
        else
        {
          env.err.println("AgentWorker.sendMessage: ERROR can't find agent: " + message.to );
          env.err.println("MESSAGE " + message + "LOST!" );
        }
      }
      else
      {
        //4) Dov' l'agente?
        env.networkManager.sendCommand( message.to.getHome(), sendMessageCommand );

        //env.err.println("AgentWorker.sendMessage: Sending " + message + " to agent's home: " + message.to.getHome() );
      }
    }
    else
    {
      env.err.println( "I have already tried " + attemptsCount + " times to deliver this message:" );
      env.err.println( "   " + message + " DISCARDED." );
    }
  }

  /** Fa le pulizie, oppure notifica la morte allo home place dell'agente.
  */
  public void agentDeath( AgentID agentID )
  {
    if( agentID.getHome().equals( env.placeID ) )
      agentPositionStore.remove( agentID );
    else
      env.networkManager.sendCommand( agentID.getHome(), new AgentDeathCommand( agentID ) );
  }

  /** Creazione di un nuovo AgentID. */
  public synchronized AgentID newAgentID()
  {
    return new AgentIDSigned( env.placeID, AgentIDCounter++ );
  }

  /** Restituisce il numero di worker e quindi di agenti del place. */
  public int agentsNumber()
  {
    return agentWorkerStore.workersNumber();
  }

  /** Impacchetta l'agente per il trasporto.
  * <P>Modificare questa funzione
  * per introdurre nuovo AgentPacket.
  */
  public AgentPacket packAgent( Agent agent )
  {
    AgentPacket agentPacket;

    if( agent.getClass().getClassLoader() instanceof AgentClassLoader )
      agentPacket = new BasicAgentPacketSecurity( agent );
    else
      agentPacket = new SystemAgentPacket( agent );

    return agentPacket;
  }

  public void loadWorkers( List list )
  {
    Iterator i = list.iterator();

    env.out.println( "AgentManager: Loading workers:" );

    for( int j = 1; i.hasNext(); j++ )
    {
      SavedWorker saved = (SavedWorker)i.next();
      try
      {
        AgentID agentID = saved.restoreWorker( env );
        env.out.println( "  " + j + ") " + agentID + " loaded!" );
      }
      catch( Exception e )
      {
        env.out.println( "  " + j + ") " + saved + ": ERROR: " + e );
      }
    }
    env.out.println();
  }

  public void load() throws IOException, ClassNotFoundException
  {
    load( env.placeID.toString() + "_agents.dat" );
  }

  public void save() throws IOException
  {
    save( env.placeID.toString() + "_agents.dat" );
  }

  public void save( String fileName ) throws IOException
  {
    ObjectOutputStream oos = new ObjectOutputStream( new FileOutputStream( new File( fileName ) ) );

    Hashtable table = new Hashtable();

    table.put( "count", new Integer( AgentIDCounter ) );
    env.out.println( "count: " + AgentIDCounter );

    table.put( "positions", agentPositionStore );
    env.out.println( "positions: " + agentPositionStore );

    table.put( "agents", agentWorkerStore.saveWorkers( env ) );
    env.out.println( "agents: DONE"  );

    oos.writeObject( table );
    oos.close();

    env.out.println( "Status saved!" );
  }

  public void load( String fileName ) throws IOException, ClassNotFoundException
  {
    ObjectInputStream ois = new ObjectInputStream( new FileInputStream( new File( fileName ) ) );

    Hashtable table = new Hashtable();

    table = (Hashtable)ois.readObject();
    ois.close();

    int newCount = ((Integer)table.get( "count" )).intValue();
    int oldCount = AgentIDCounter;

    if( newCount > oldCount )
    {
      AgentIDCounter = newCount;
      env.out.println( "agent counter:" + oldCount + " -->" + newCount );
    }

    agentPositionStore = (AgentPositionStore)table.get( "positions" );
    env.out.println( "positions: " + agentPositionStore );

    loadWorkers( (List)table.get( "agents" ) );
    env.out.println( "agents: DONE"  );

    env.out.println( "Status loaded!" );
  }


  public String toString()
  {
    return "[agentManager: " + agentsNumber() + " agents]";
  }
}
